/*
	$Id: input.h,v 1.5 2000/07/15 15:45:46 starch Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

//! component="Input"

#ifndef header_input
#define header_input

#include <vector>
#include "../../signals.h"

#include "key.h"

class CL_InputDevice;
class CL_Keyboard;

class CL_Input
//: Input component class.
// CL_Input is the access point to keyboards, joysticks and (mouse) pointers.
{
public:
	virtual ~CL_Input() { return; }

	static std::vector<CL_Keyboard *> keyboards;
	//: List of keyboards available.
	
	static std::vector<CL_InputDevice *> joysticks; // joysticks and gamepads
	//: List of joysticks and gamepads available.
	
	static std::vector<CL_InputDevice *> pointers; // mice and digitizers
	//: List of mice and digitizers available.

	static thSignal_v2<CL_InputDevice *, const CL_Key &> sig_button_press;
	//: Signal signaled upon button press.

	static thSignal_v2<CL_InputDevice *, const CL_Key &> sig_button_release;
	//: Signal signaled upon button release.

	static thSignal_v3<CL_InputDevice *, int, int> sig_mouse_move;
	//: Signal signaled upen mouse move.
};

#endif

