/*
	Dodge This, ClanLib example Game,
	Copyright 1999-2000 by the Dodge This Development Team.
	
	This game is licensed under the GNU PUBLIC LICENSE (GPL), version 2.
*/

#ifndef header_provider_3ds
#define header_provider_3ds

#include "../../Core/Math/cl_vector.h"
#include "../../Core/IOData/inputsource_provider.h"
#include "object3d_provider.h"
#include "object3d.h"


 
class CL_3DSProvider : public CL_Object3DProvider
//: 3D Object representation that loads objects from the 3D Studio format.
// <p>Use this class load to .3ds files. It is very primitive, so hopefully
// it can read the object without problems.</p>
{
public:
	static CL_Object3D *create(
		std::string handle,
		CL_Vector init_scale,
		CL_Vector init_pos,
		CL_Vector init_dir,
		CL_Vector init_up,
		CL_InputSourceProvider *provider = NULL
		);


	
	CL_3DSProvider(
		std::string handle,
	/*	CL_Vector init_scale,
		CL_Vector init_pos,
		CL_Vector init_dir,
		CL_Vector init_up,
	*/	CL_InputSourceProvider *provider = NULL);
	// Constructs the 3D object from a file, loaded from the input
	// source provider if specified.
	//!param: filename - Identifier / filename
	//!param: provider - The input source provider to load from, or NULL if
	//!param:            to be loaded from file.
	

	static CL_Object3D *load(
		CL_InputSource *input,
		CL_ResourceManager *manager);
	// Loads the object from a datafile input source.
/*
	CL_Object3D_3DS(
		CL_InputSource *input,
		CL_ResourceManager *manager);
	// Loads the object from a datafile input source.

	CL_Object3D_3DS(
		CL_InputSource *input,
		CL_Vector init_scale,
		CL_Vector init_pos,
		CL_Vector init_dir,
		CL_Vector init_up,
		CL_ResourceManager *manager);
	// Loads the object from a file input source.
*/
	virtual ~CL_3DSProvider();
/*	
	virtual void show(
		const Viewpoint &viewpoint,
		ClipInfo &clipinfo,
		float scale = 1.0f);
*/

	
//	void load();		//perform_lock();
//	void unload();		//perform_unlock();





// implementation
private:
	void init();

	void handle_chunks(CL_InputSource *input/*, int depth, int size*/);

	std::string read_str(CL_InputSource *input);
//	read_color_chunk(CL_InputSource *input);




	CL_InputSourceProvider *provider;



	// tmp data
	std::string						tmp_string;
	Object3D_Data_TriangularMesh	tmp_trimesh;
	Object3D_Data_Vertex			tmp_vertex;
	Object3D_Data_Face              tmp_face;

	// chunk fucations
	void ObjBlockReader(CL_InputSource *input);
	void TriangularMeshReader(CL_InputSource *input);
	void VertListReader (CL_InputSource *input);
	void FaceListReader (CL_InputSource *input);
	
//	void ChunkReader(CL_InputSource *input);
//	void SkipReader (CL_InputSource *input, ChunkHeader_3DS chunk_h);
/*	void RGBFReader (CL_InputSource *input);
	void RGBBReader (CL_InputSource *input);
	void RGBFGammaReader (CL_InputSource *input);
	void RGBBGammaReader (CL_InputSource *input);
	void WRDReader (CL_InputSource *input);
	void FloatReader (CL_InputSource *input);
	void SHTReader (CL_InputSource *input);
	int  ASCIIZReader (CL_InputSource *input);
	void TriangularMeshReader(CL_InputSource *input);
	int  VertListReader (CL_InputSource *input);
	int  FaceListReader (CL_InputSource *input, int chunk_len);
	int  FaceMatReader (CL_InputSource *input, int chunk_len);
	void MapListReader (CL_InputSource *input);
	void SmooListReader (CL_InputSource *input, int chunk_len);
	void TrMatrixReader(CL_InputSource *input);
	void LightReader(CL_InputSource *input);
	void SpotLightReader(CL_InputSource *input);
	void SpotRectReader(CL_InputSource *input);
	void SpotMapReader(CL_InputSource *input);
	void SpotRollReader(CL_InputSource *input);
	void CameraReader(CL_InputSource *input);
	void MeshVersionReader(CL_InputSource *input);
	void MatEditReader(CL_InputSource *input);
	void MatNameReader (CL_InputSource *input);
	void MatAmbientReader(CL_InputSource *input);
	void MapFileReader(CL_InputSource *input);
	void TextFlagReader (CL_InputSource *input);
	void FramesReader(CL_InputSource *input);
	void TrackObjNameReader(CL_InputSource *input);
	void PivotPointReader(CL_InputSource *input);
	// NOTE THIS IS NOT A CHUNK, but A PART OF SEVERAL CHUNKS
	void SplineFlagsReader(CL_InputSource *input, unsigned short flags);
	void TrackPosReader(CL_InputSource *input);
	void TrackRotReader(CL_InputSource *input);
	void TrackScaleReader(CL_InputSource *input);
	void TrackFovReader(CL_InputSource *input);
	void TrackRollReader(CL_InputSource *input);
	void TrackMorphReader(CL_InputSource *input);
	void TrackHideReader(CL_InputSource *input);
	void ObjNumberReader(CL_InputSource *input);
	void FindAndRunChunk(CL_InputSource *input);
	void TDSVersionReader(CL_InputSource *input);
*/
};


enum {
    CHUNK_RGBF	    = 0x0010,
    CHUNK_RGBB	    = 0x0011,
	CHUNK_RGBB_GAMMA= 0x0012,
	CHUNK_RGBF_GAMMA= 0x0013,
    CHUNK_WRD	    = 0x0030,
    CHUNK_PRJ	    = 0xC23D,
    CHUNK_MLI	    = 0x3DAA,
    CHUNK_SHP	    = 0x2D2D,
    CHUNK_LFT	    = 0x2D3D,
    CHUNK_MAIN	    = 0x4D4D,
	CHUNK_VERSION   = 0x0002,
    CHUNK_3DEDITOR	= 0x3D3D,
		CHUNK_ONEUNIT   = 0x0100,
        CHUNK_BKGCOLOR  = 0x1200,
        CHUNK_AMBCOLOR  = 0x2100,
        CHUNK_OBJBLOCK  = 0x4000,
      CHUNK_TRIMESH	= 0x4100,
          CHUNK_VERTLIST  = 0x4110,
          CHUNK_VERTFLAGS = 0x4111,
          CHUNK_FACELIST  = 0x4120,
            CHUNK_FACEMAT   = 0x4130,
          CHUNK_MAPLIST   = 0x4140,
          CHUNK_SMOOLIST  = 0x4150,
          CHUNK_TRMATRIX  = 0x4160,
          CHUNK_MESHCOLOR = 0x4165,
          CHUNK_TXTINFO   = 0x4170,
      CHUNK_LIGHT	= 0x4600,
          CHUNK_LIGHTR1  = 0x4659,
          CHUNK_LIGHTR2  = 0x465A,
          CHUNK_LIGHTMUL = 0x465B,
          CHUNK_SPOTLIGHT = 0x4610,
			  CHUNK_SPOT_RECT = 0x4651,
			  CHUNK_SPOT_MAP  =	0x4653,
			  CHUNK_SPOT_ROLL = 0x4656,
      CHUNK_CAMERA	= 0x4700,
      CHUNK_HIERARCHY = 0x4F00,
    CHUNK_VIEWPORT = 0x7001,
	CHUNK_MESH_VERSION = 0x3D3E,
    CHUNK_MATERIAL	       = 0xAFFF,
        CHUNK_MATNAME      = 0xA000,
        CHUNK_AMBIENT      = 0xA010,
        CHUNK_DIFFUSE      = 0xA020,
        CHUNK_SPECULAR     = 0xA030,
        CHUNK_SHININESS    = 0xA040,
        CHUNK_SHSTRENGTH   = 0xA041,
        CHUNK_TRANSP       = 0xA050,
        CHUNK_TRFALLOFF    = 0xA052,
        CHUNK_REFLECTBL    = 0xA053,
        CHUNK_FLAG2SIDE    = 0xA081,
        CHUNK_FLAGADD      = 0xA083,
        CHUNK_SELFILLUM    = 0xA084,
        CHUNK_WIREFRAME    = 0xA085, // Flag
        CHUNK_WIRETHICK    = 0xA087, // Float
        CHUNK_FACEMAP      = 0xA088, // Flag
        CHUNK_FLAGIN       = 0xA08A,
        CHUNK_SOFTEN       = 0xA08C, // Flag
        CHUNK_FLAGUNIT     = 0xA08E, // Very useless
        CHUNK_SHADETYPE    = 0xA100,
        CHUNK_TEXTURE      = 0xA200,
        CHUNK_TEXTUREMASK  = 0xA33E,
        CHUNK_TEXTURE2     = 0xA33A,
        CHUNK_TEXTUREMASK2 = 0xA340,
        CHUNK_REFMAP       = 0xA220,
        CHUNK_REFMASK      = 0xA34C,
        CHUNK_OPACITYMAP   = 0xA210,
        CHUNK_OPACITYMASK  = 0xA342,
        CHUNK_SPECULARMAP  = 0xA204,
        CHUNK_SPECULARMASK = 0xA348,
        CHUNK_SHININESSMAP = 0xA33C,
        CHUNK_SHININESSMASK= 0xA346,
        CHUNK_SELFILLUMMAP = 0xA33D,
        CHUNK_SELFILLUMMASK= 0xA34A,
        CHUNK_BUMPMAP      = 0xA230,
        CHUNK_BUMPMASK     = 0xA344,
        CHUNK_MAPFILE      = 0xA300,
          CHUNK_TFLAGS     = 0xA351, // Flags
          CHUNK_BLUR       = 0xA353, // Float
          CHUNK_USCALE     = 0xA354, // Float
          CHUNK_VSCALE     = 0xA356, // Float
          CHUNK_UOFFSET    = 0xA358, // Float
          CHUNK_VOFFSET    = 0xA35A, // Float
          CHUNK_TROTATION  = 0xA35C, // Float
          CHUNK_BLACKTINT  = 0xA360, // Float
          CHUNK_WHITETINT  = 0xA362, // Float
          CHUNK_RTINT      = 0xA364, // Float
          CHUNK_GTINT      = 0xA366, // Float
          CHUNK_BTINT      = 0xA368, // Float
    CHUNK_KEYFRAMER = 0xB000,
        CHUNK_AMBIENTKEY	= 0xB001,
        CHUNK_TRACKINFO = 0xB002,
      CHUNK_TRACKOBJNAME  = 0xB010,
      CHUNK_TRACKPIVOT    = 0xB013,
      CHUNK_TRACKPOS	    = 0xB020,
      CHUNK_TRACKROTATE   = 0xB021,
      CHUNK_TRACKSCALE    = 0xB022,
      CHUNK_TRACKMORPH    = 0xB026,
      CHUNK_TRACKHIDE     = 0xB029,
      CHUNK_OBJNUMBER     = 0xB030,
        CHUNK_TRACKCAMERA = 0xB003,
      CHUNK_TRACKFOV	= 0xB023,
      CHUNK_TRACKROLL = 0xB024,
        CHUNK_TRACKCAMTGT = 0xB004,
        CHUNK_TRACKLIGHT  = 0xB005,
        CHUNK_TRACKLIGTGT = 0xB006,
        CHUNK_TRACKSPOTL  = 0xB007,
        CHUNK_FRAMES    = 0xB008,
    CHUNK_OBJMESH	= 0x3D3D
  };

#endif


