//! component="GUI"
/*
	ClanGUI, copyrights by various people. Have a look in the CREDITS file.
	
	This sourcecode is distributed using the Library GNU Public Licence,
	version 2 or (at your option) any later version. Please read LICENSE
	for details.
*/

#ifndef header_button
#define header_button

#include "../Core/Math/point.h"
#include "component.h"
#include "component_options.h"
#include "stylemanager.h"

class CL_Button : public CL_Component
//: Button component
{
public:
	static CL_Button *create(
		const CL_ComponentOptions &options,
		CL_Component *parent,
		CL_StyleManager *style);

	static CL_Button *create(
		const CL_Rect &pos,
		std::string label,
		CL_Component *parent,
		CL_StyleManager *style = NULL);

	static CL_Button *create(
		const CL_Point &pos,
		std::string label,
		CL_Component *parent,
		CL_StyleManager *style = NULL);

	CL_Button(const CL_ComponentOptions &options, CL_Component *parent);
	CL_Button(CL_Component *parent) : CL_Component(parent) { ; }

	virtual ~CL_Button() { return; }

	void set_text(const std::string &text) { this->text = text; }
	//: Sets the button label.
	//!param: text - The button label.

	const std::string &get_text() { return text; };
	//: Returns the button label.
	//!retval: The button label.

	bool get_state() const { return state; }
	//: Returns current state of button.
	//!retval: true if button is pressed down, false otherwise.

	void set_state(bool new_state) { if (new_state != state) { state = new_state; sig_toggled(state); } }
	//: Sets the state of button.
	//!param: bool new_state - true for pressed down.

	void toggle() { state = !state; sig_toggled(state); };
	//: Toggles the button.

	void set_sticky_mode(bool sticky = true) { sticky_mode = sticky; };
	bool get_sticky_mode() { return sticky_mode; };
	
	thSignal_v1<bool> sig_toggled;
	//: This signal is emitted whenever a toggle button changes status.
	//!param: bool on - Is true if the button is on, or false otherwise.

	thSignal_v0 sig_clicked;
	//: This signal is emitted when the button is activated (i.e. first pressed down and then released when the mouse cursor is inside the button).
	
	thSignal_v0 sig_pressed;
	//: This signal is emitted when the button is pressed down. 

	thSignal_v0 sig_released;
	//: This signal is emitted when the button is released.
	
	std::string text;
	bool state;
	bool captured;
	bool outside;
	bool sticky_mode;

private:
	virtual void on_key_down(CL_Component *comp, CL_InputDevice *device, CL_Key key);
	virtual void on_key_up(CL_Component *comp, CL_InputDevice *device, CL_Key key);
	virtual void on_mouse_move(CL_Component *comp, CL_InputDevice *device, int x, int y);

	thSlot slot_key_down;
	thSlot slot_key_up;
	thSlot slot_mouse_move;
};  

#endif

