/*
	ClanGUI, copyrights by various people. Have a look in the CREDITS file.
	
	This sourcecode is distributed using the Library GNU Public Licence,
	version 2 or (at your option) any later version. Please read LICENSE
	for details.
*/

#ifndef file_dialog_header
#define file_dialog_header

#ifndef WIN32
#include <dirent.h>
#endif
#include "component.h"
#include "component_options.h"
#include "stylemanager.h"
#include "button.h"
#include "inputbox.h"
#include "listbox.h"

class CL_FileDialog : public CL_Component
{
public:
	static CL_FileDialog *create(
		const CL_ComponentOptions &options,
		CL_Component *parent,
		CL_StyleManager *style);

	static CL_FileDialog *create(
		CL_Component *parent,
		CL_StyleManager *style);

	CL_FileDialog(const CL_ComponentOptions &options, CL_Component *parent, CL_StyleManager *style);
	CL_FileDialog(CL_Component *parent) : CL_Component(parent) { ; }
	
	virtual ~CL_FileDialog() { return; }

	void update_files();
	int select_files(const struct dirent *);

	void set_path(CL_String new_path)
	{
		path = new_path;
		input_file->set_value(path);
		update_files();
	}

	void set_ext(CL_String new_ext)
	{
		ext = new_ext;
		update_files();
	}

	thSignal_v1<CL_String> sig_ok;
	thSignal_v0 sig_cancel;	

protected:
	void on_ok();
	void on_cancel();
	void on_list(int);

//	thSlotContainer slots;
	thSlot slot_ok;
	thSlot slot_cancel;
	thSlot slot_list;

	CL_String filename;
	CL_String path;
	CL_String ext;
	
	bool must_exist;

	CL_Button *button_ok;
	CL_Button *button_cancel;
	CL_Inputbox *input_file;
	CL_Listbox *list_dir;

	CL_StyleManager *style;
};

#endif

