/*
	ClanGUI, copyrights by various people. Have a look in the CREDITS file.
	
	This sourcecode is distributed using the Library GNU Public Licence,
	version 2 or (at your option) any later version. Please read LICENSE
	for details.
*/

#ifndef header_listbox
#define header_listbox

#include "component.h"
#include "component_options.h"
#include "stylemanager.h"

class CL_Scrollbar;

class CL_Listbox : public CL_Component
{
	friend class CL_Listbox_Default;
public:
	static CL_Listbox *create(
		const CL_ComponentOptions &options,
		CL_Component *parent,
		CL_StyleManager *style);
	
	static CL_Listbox *create(
		const CL_Rect &pos,
		CL_Component *parent,
		CL_StyleManager *style = NULL);

	CL_Listbox(const CL_ComponentOptions &options, CL_Component *parent,
		   CL_StyleManager *style);
	CL_Listbox(CL_Component *parent);

	virtual ~CL_Listbox() { return; }

	unsigned int count() const;

	int insert_item(std::string item, int index = -1);
	void remove_item(int index);

	void clear();

	int get_current_item();
	void set_current_item(int index);
	std::string get_item_data(int item);
	std::string get_current_item_data();

	thSignal_v1<int> sig_selected;

protected:
	void on_key_down(CL_Component *comp, CL_InputDevice *device, CL_Key key);
	
	int highlighted_option;
	int font_height;

	bool has_scrollbar;
	unsigned int max_height;
	int scroll_offset;
	int contents_y_offset;
	void on_scroll_change(int new_offset);
	CL_Scrollbar *scrollbar;

	std::list<std::string> selection_list;
	thSlotContainer slots;

private:
	void update_scrollbar();
};

#endif

