/*
	ClanGUI, copyrights by various people. Have a look in the CREDITS file.
	
	This sourcecode is distributed using the Library GNU Public Licence,
	version 2 or (at your option) any later version. Please read LICENSE
	for details.
*/

#ifndef header_menudata
#define header_menudata

#include "component.h"

#include <string>

class CL_MenuItem;
class CL_PopupMenu;
class CL_Surface;

class CL_MenuData : public CL_Component
{
public:
	CL_MenuData(CL_Component *parent);
	virtual ~CL_MenuData();

	int insert_item(const std::string &text, int id = -1, int index = -1);
	int insert_separator(int id = -1, int index = -1);

	unsigned int count() const;
	// Return number of items

	void remove_item(int id);
	// Remove item

	void remove_item_at(int index);
	// Remove item as position index

	void clear();
	// Remove all items

	const std::string &get_text(int id) const;
	// Return item text

	void change_item(const std::string &text, int id);
	// Change item text

	bool is_item_enabled(int id) const;
	void enable_item(int id, bool enable);
	bool is_item_checked(int id) const;
	void check_item(int id, bool check);

	int get_index(int id) const;
	// Returns item position

	int get_id(int index) const;
	// Returns item id

	void set_id(int index, int id);
	// Sets item id

protected:
	virtual void on_item_activated(CL_MenuItem *item, CL_PopupMenu *child) {};
	virtual void on_item_highlighted(CL_MenuItem *item) {};

	CL_Component *real_parent;

private:
	int generate_id(int id);
	// Returns a new available ID
	// Doesn't generate a new id if (id == -1)
};  

#endif

