/*
	ClanGUI, copyrights by various people. Have a look in the CREDITS file.
	
	This sourcecode is distributed using the Library GNU Public Licence,
	version 2 or (at your option) any later version. Please read LICENSE
	for details.
*/

#ifndef header_menuitem
#define header_menuitem

#include "../Core/System/timer.h"

#include "component.h"
#include "component_options.h"
#include "stylemanager.h"

class CL_PopupMenu;

class CL_MenuItem : public CL_Component
{
public:
	static CL_MenuItem *create(
		const CL_ComponentOptions &options,
		CL_Component *parent,
		CL_StyleManager *style);

	CL_MenuItem(const CL_ComponentOptions &options, CL_Component *parent);
	virtual ~CL_MenuItem() { return; }

	virtual void add_child(CL_Component *child);

//	virtual void update();

	bool is_checkable() { return checkable; }
	bool is_separator() { return separator; }
	bool is_checked() { return checked; }
	bool is_highlighted() { return highlighted; }

	bool has_submenu() { return false; }
	
	const std::string &get_text() { return text; }
	//: Returns the item label.
	//!retval: The item label.

	void set_text(const std::string &text) { this->text = text; }
	//: Sets the item label.
	//!param: text - The item label.

	thSignal_v0 sig_clicked;

	void set_impl(CL_ComponentStyle *impl) { this->impl = impl; }
	CL_ComponentStyle *get_impl() { return impl; }
	
private:
	void init(int id);
	void close_children();
	
protected:
	void on_mouse_enter();
	void on_mouse_leave();

	void on_key_down(CL_Component *comp, CL_InputDevice *device, CL_Key key);
	void on_timer_popup();

	thSlotContainer slots;

protected:
	int id;
	
	std::string text;
	
	bool separator;
	bool checked;
	bool checkable;
	bool highlighted;

	CL_PopupMenu *submenu;
	
	CL_Timer timer_popup;

	CL_ComponentStyle *impl;
};

#endif

