/*
	ClanGUI, copyrights by various people. Have a look in the CREDITS file.
	
	This sourcecode is distributed using the Library GNU Public Licence,
	version 2 or (at your option) any later version. Please read LICENSE
	for details.
*/

#ifndef header_scrollbar
#define header_scrollbar

#include "../Core/System/timer.h"

#include "component.h"
#include "component_options.h"
#include "stylemanager.h"

class CL_Button;

class CL_Scrollbar : public CL_Component
{
	friend class CL_Scrollbar_Default;
public:
	static CL_Scrollbar *create(
		const CL_ComponentOptions &options,
		CL_Component *parent,
		CL_StyleManager *style);

	static CL_Scrollbar *create(
		const CL_Rect &pos,
		bool orientation,
		CL_Component *parent,
		CL_StyleManager *style = NULL);

	CL_Scrollbar(const CL_ComponentOptions &options, CL_Component *parent);
	CL_Scrollbar(CL_Component *parent) : CL_Component(parent), scroll_timer(250) { ; }

	virtual ~CL_Scrollbar() { return; }

	int range() { return scroll_range; }
	void set_range(int new_range);
	void set_cur_position(int new_pos);
	int get_cur_position() { return cur_position; }

	thSignal_v1<int> sig_scrolled;

protected:
	int scroll_range;
	int cur_position;
	bool vertical;
	bool mouse_captured;
	bool scrolling;
	bool section_scrolling;
	int scroll_direction;

	int drag_offset_x, drag_offset_y;
	CL_Rect drag_rect;

	void set_decrease_button(CL_Button *button);
	void set_increase_button(CL_Button *button);

private:
	CL_Button *decrease_buttton;
	CL_Button *increase_button;
	CL_Timer scroll_timer;

	int capture_orgpos;
	int capture_org_drag_offset_x, capture_org_drag_offset_y;
	int capture_x_offset, capture_y_offset;
	int last_capture_x, last_capture_y;

	thSlot decrease_slot;
	thSlot increase_slot;
	thSlot timer_slot;
	thSlot keydown_slot, keyup_slot, mousemove_slot, resize_slot;
	virtual void on_key_down(CL_Component *comp, CL_InputDevice *device, CL_Key key);
	virtual void on_key_up(CL_Component *comp, CL_InputDevice *device, CL_Key key);
	virtual void on_mouse_move(CL_Component *comp, CL_InputDevice *device, int x, int y);
	void on_scroll_timer();
	void on_decrease_pressed();
	void on_increase_pressed();
	void on_decrease_released();
	void on_increase_released();
	void on_resize(int old_width, int old_height);
};

#endif

