//! component="GUI"
/*
	ClanGUI, copyrights by various people. Have a look in the CREDITS file.
	
	This sourcecode is distributed using the Library GNU Public Licence,
	version 2 or (at your option) any later version. Please read LICENSE
	for details.
*/

#ifndef header_stylemanager_default
#define header_stylemanager_default

#include "stylemanager.h"
#include "component_options.h"

class CL_ResourceManager;
class CL_Component;

enum EGUIColor
{
	GUICOL_DEFAULT, // default, grey (0.6, 0.6, 0.6)
	GUICOL_BRIGHT_OUTLINE, // default, white (1.0, 1.0, 1.0)
	GUICOL_DARK_OUTLINE, // default, black (0.0, 0.0, 0,0)
	GUICOL_TOP_SHADE, // default, dark grey (0.5, 0.5, 0.5)
	GUICOL_BOTTOM_SHADE, // default, grey (0.5, 0.5, 0.5)
	GUICOL_TOGGLED, // default, bright grey (0.77, 0.77, 0.77)
	GUICOL_DARK_BG, // default, very dark grey (0.2, 0.2, 0.2)
	GUICOL_TEXT_BG, // default, white (1.0, 1.0, 1.0)
	GUICOL_TEXT_DISABLED_FG, // default, grey (0.5, 0.5, 0.5)
	GUICOL_TEXT_DISABLED_BG, // default, white (1.0, 1.0, 1.0)
	GUICOL_CONTOUR, // default, semi-grey rgb(212, 208, 200)
	GUICOL_SELECTION, // default, bright grey (0.5, 0.5, 0.9, 0.5)
	GUICOL_CARET, // default, black (0.0, 0.0, 0.0)
	GUICOL_SCROLL_BG // default, rgb(234, 232, 228)
};

class CL_StyleManager_Default : public CL_StyleManager
//: Default Manager
{
public:
	CL_StyleManager_Default(CL_ResourceManager *resources);
	virtual ~CL_StyleManager_Default();

// Attributes:
	CL_ResourceManager *get_resources() const;

	const std::string &get_default_font() const;

	const std::string &get_combo_normal() const;
	const std::string &get_combo_toggled() const;
	const std::string &get_combo_disabled() const;

	const std::string &get_checkbox_checked() const;
	const std::string &get_checkbox_unchecked() const;
	const std::string &get_checkbox_disabled() const;

	bool uses_solid_background() const;
	const std::string &get_background_texture() const;

// Operations:
	void set_default_font(const std::string &font);

	void set_combo_normal(const std::string &normal);
	void set_combo_toggled(const std::string &toggled);
	void set_combo_disabled(const std::string &disabled);

	void set_checkbox_checked(const std::string &checked);
	void set_checkbox_unchecked(const std::string &unchecked);
	void set_checkbox_disabled(const std::string &disabled);

	void set_background_mode(bool use_solid_background);
	void set_background_texture(const std::string &background_texture);

// Overrideables:
	virtual CL_Component *create_component(
		const std::string &type,
		const CL_ComponentOptions &options,
		CL_Component *parent);

// Functions:
	void fill_rect(int x1, int y1, int x2, int y2, EGUIColor col);
	void draw_line(int x1, int y1, int x2, int y2, EGUIColor col);
	void draw_box(int x1, int y1, int x2, int y2, EGUIColor topleft, EGUIColor bottomright);
	
// Implementation:
private:
	CL_StyleManager_Default(const CL_StyleManager_Default &copy) { return; } // disallow copy contruction
	class CL_StyleManager_Default_Generic *impl;
};

#endif

