/** \file
 * For internal use only
 *
 * The Herd (Library)
 * Copyright (C) 1999 Will Henderson
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Information about the LGPL can be found on the GNU web site,
 *  http://www.gnu.org/copyleft/lesser.html
 * 
 * This software is OSI Certified Open Source Software.
 * OSI Certified is a certification mark of the Open Source Initiative.
 * 
 * Information on the Open Source Initiative can be found at
 *  http://www.opensource.org
 * 
 * Further information about this library and how to contact
 * its author can be found at
 *  http://whenderson.cjb.net
 *
 */


#ifndef _TheHerd_STD_STD_H_
#define _TheHerd_STD_STD_H_

// contains a link to the platform specific include

/* mbn: always default not to use the namespace.
#ifdef _MSC_VER
  #include "TheHerd/std/msvc/defines.h"
#else
  #include "TheHerd/std/default/defines.h"
#endif
*/

#ifdef thUSE_NAMESPACE
  #define thNAMESPACE_begin(x) namespace x {
  #define thNAMESPACE_end       };
  #define thUSING(x) using x
  #define thNS(x) x
#else
  #define thNAMESPACE_begin(x)
  #define thNAMESPACE_end
  #define thUSING(x)
  #define thNS(x) x
#endif

#define thBEGIN\
  thNAMESPACE_begin(TheHerd)
#define thEND\
  thNAMESPACE_end;

// use these try/catch blocks to create a 'finaly' block for your try/catch blocks.
// the 'finaly' block will be called regardless of weather an exception was thrown,
// and any exceptions thrown will continue on through the block regardless.
// If the finaly block throwns an exception however, the original exception will be 
// disregarded

thBEGIN
  struct th_finaly_ex {
   };
thEND

#define th_try\
  { bool __th_exception = true;\
    try {\
      try {
#define th_catch(x)\
      }\
      catch (x) {
#define th_finaly\
      }\
      __th_exception = false;\
      throw th_finaly_ex;\
    }\
    catch (...) {
#define th_end\
      if (__th_exception)\
        throw;\
    }\
  }
/*
  // example
  th_try
    // do your stuff
  th_catch(some exception type)
    // do your stuff
  th_catch(some other exception type)
    // do your stuff
  th_finaly
    // do your cleanup
  th_end
*/
#endif

